;
; ACL2 Windows Installer for NSIS Installer System
; Copyright (C) 2005-2008 by Jared Davis <jared@cs.utexas.edu>
; Updated for ACL2 3.1 by Alex Spiridonov <alexsp@cs.utexas.edu>
;
; This program is free software; you can redistribute it and/or modify it under
; the terms of the GNU General Public License as published by the Free Software
; Foundation; either version 2 of the License, or (at your option) any later
; version. This program is distributed in the hope that it will be useful but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
; more details. You should have received a copy of the GNU General Public
; License along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;
; Notes 
;
;   - I am using NSIS Version 2.38.  The NSIS home page is:
;   http://nsis.sourceforge.net/
;
;   - To create the installer, you will also need the files path.nsh and 
;   WriteEnvStr.nsh, which were each taken from the NSIS archive and are 
;   presumably public domain or GPL or otherwise fair for to use.
;
; Brief Changelog
;
;   - I began providing these installers for ACL2 2.8.
;
;   - For ACL2 2.8-2.9, I provided "net installers" that allowed you to
;   download a small installer which would download only the required files.  I
;   decided for installers since then that it would be better if everything was
;   less complicated, and so this functionality has been removed.  Hence, this
;   script will only allow you to create a "full installer", which can be quite
;   large but still should fit on a CD and be an easy download using a cable
;   modem.
;
;   - For ACL2 2.8-2.9.4, I provided an acl2r image along with the installer.
;   Starting with ACL2 3.0, I have since discontinued this practice, since it 
;   was a lot of extra space and took some extra work.
;
;   - For ACL2 3.4, I added SBCL as an alternate Lisp.  I would like to provide
;   a hons image eventually, but it's complaining about my 32-bit system for
;   now so that'll have to wait.  I also removed the copyright notice since (1)
;   not everything is under the GPL anymore (since SBCL is apparently public
;   domain), and (2) it's all a matter of copyright anyway, not license
;   agreements, so there is nothing for the user to "agree" to, they're bound
;   by it whether they like it or not.
;
; 
; Creating An Updated Installer
;
;   - When it comes time to update this installer, there are two paths one 
;   may take.
;
;     1.  (Easy) Install the current ACL2 distribution, delete the acl2-sources
;     directory and the ACL2 executables, perhaps update your GCL/SBCL
;     versions, build the new versions of ACL2 using the existing compiler and
;     certify all the books, then finally create an installer.
;
;     2.  (Hard) Start from scratch, installing MSYS and MINGW, then
;     compile/install your Lisp images, ACL2, certify the books, arrange it
;     all and create an installer.
;
;   - The reason 2 is harder is that you have to decide which programs to use
;   from msys or cygwin or whatever you go with.  But it might be worthwhile to
;   update from time to time, since otherwise our gcc version and such will be
;   falling behind.
;
;   - In either case, you should try to mimic the directory structure that the
;   existing distribution provides.  In other words, include:
;
;     ...\ACL2-X.Y\bin
;     Contains the basic UNIX programs like gcc and make, and also contains
;     the ACL2, GCL, and SBCL executables.
;
;     ...\ACL2-X.Y\emacs-X.Y  
;     Contains a copy of the EMACS text editor program
;
;     ...\ACL2-X.Y\etc
;     Contains the MSYS/MINGW etc files
;
;     ...\ACL2-X.Y\lib
;     Contains the GCL lib files
;
;     ...\ACL2-X.Y\mingw
;     Contains the MINGW files (bin, include, lib, mingw32, ...)
;
;     ...\ACL2-X.Y\acl2-sources
;     Contains the ACL2 source code and books
;
;   - The top level folder also has the ACL2 icon and so forth.
;
;   - Once you have a working, updated distribution, all you need to do is
;   "installerify" it.  This basically consists of splitting it up into the
;   appropriate directories and then running this installer script through the
;   NSIS program.  
;
;   - You will need to create the following directory structure (where "top"
;   exists somewhere on your system and can be named anything you want, e.g.,
;   maybe it is named "ACL2 Installer")

;       
;        /top/ACL2.nsi             a copy of this file
;
;        /top/path.nsh             a copy of the NSIS libraries we're using
;        /top/WriteEnvStr.nsh 
;
;        /top/acl2.ico             a copy of the ACL2 icon
;
;        /top/license.txt          the copyright information file
;
;        /top/post-install.bat     a wrapper for the post-install.sh file
;
;        /top/post-install.sh      a post install script that goes through all 
;                                  the books and "touches" the .o/fasl files, so
;                                  that include-book commands will not think 
;                                  that files are out of date
;
;        /top/dist/                a new folder that will contain most of the
;                                  distribution files 
;
;        /top/dist/bin/            a copy of the "bin" folder that holds all of
;                                  the UNIX and ACL2/GCL executables
;
;        /top/dist/books           a copy of the acl2-sources/books folder with 
;                                  all of the books certified, including the .o
;                                  and .cert files, but NOT the workshop books.
;
;        /top/dist/emacs-21.3/     a copy of the GNU Emacs text editor files
;
;        /top/dist/etc/            a copy of the MSYS/MINGW etc directory
;
;        /top/dist/lib/            a copy of the GCL "lib" directory
;
;        /top/dist/mingw/          a copy of the MSYS/MINGW mingw directory
;
;        /top/dist/acl2-sources/   the ACL2 sources directory, but without 
;                                  including the books and workshop books!
;
;        /top/dist/workshops/      a copy of all the workshop books with their
;                                  .cert and .o files already created
;
;   - After creating all these folders and copying the right files into them,
;   you should edit this ACL2.nsi file by going down to the "Configuration"
;   section below, and filling in the appropriate variables.
;
;   - Then all you need to do is launch the MakeNSISW program (a gui to the
;   compiler), select your compression type, then drag and drop ACL2.nsi onto
;   the window.  It will chug for awhile and create an installer.
;
;   - I recommend using the default compression for testing, and the LZMA
;   compression to create the final installer.  LZMA is MUCH better
;   compression, but it takes MUCH longer to create the installer.



; Preliminaries
;
; We now begin the code for the installer itself.  We will use the MUI file,
; which instructs NSIS to create a "modern" looking installer.  We also use the
; PATH.nsh file, which gives us functions for adding our bin directory to the
; user's path.  We also write a simple macro to create "Internet shortcuts" to
; web sites, such as the ACL2 homepage, in the user's start menu.

!include "MUI.nsh"
!include "path.nsh"
!include "WriteEnvStr.nsh"

!macro CreateInternetShortcut FILENAME URL ICONFILE ICONINDEX
  WriteINIStr "${FILENAME}.url" "InternetShortcut" "URL" "${URL}"
  WriteINIStr "${FILENAME}.url" "InternetShortcut" "IconFile" "${ICONFILE}"
  WriteINIStr "${FILENAME}.url" "InternetShortcut" "IconIndex" "${ICONINDEX}"
!macroend



; Configuration
;
; These are the parameters that you would likely want to change.  First, what
; version of ACL2 are we installing?  Also, what should be the name of the 
; installer (for its title bar), the name of the installer's executable file,
; and the default directory to install to?

!define VER "3.4"
!define WebRoot    "http://www.cs.utexas.edu/users/moore/acl2/v3-4/"

Name       "ACL2 for Windows, Version ${VER}"	; window title
InstallDir "C:\Program Files\ACL2 ${VER}"	; default directory
OutFile    "ACL2 ${VER} Installer.exe"          ; installer name to create



; We now set up the installer's page sequence, i.e., what steps will the
; installer lead the user through before finishing?

!define MUI_ABORTWARNING                   ; warn on early cancel
; !insertmacro MUI_PAGE_LICENSE "gpl.txt"    ; first display the copyright
!insertmacro MUI_PAGE_COMPONENTS           ; then user chooses components
!insertmacro MUI_PAGE_DIRECTORY            ; then user chooses install dir
!insertmacro MUI_PAGE_INSTFILES            ; then we install the files
!insertmacro MUI_UNPAGE_CONFIRM            ; uninstaller: user must confirm first
!insertmacro MUI_UNPAGE_INSTFILES          ; then uninstall the files
!insertmacro MUI_LANGUAGE "English"        ; installer is in english




; Sections
;
; We now define several sections.  Sections whose names begin with - are
; mandatory and cannot be unselected.  Sections whose names begin with ! are
; merely recommended, but optional.  Sections given with /o are unselected
; by default, but may be chosen by the user later.

Section "- Minimal Files" SecMinimal   

  SetOutPath "$INSTDIR"     ; copy files to the install folder dir directly

  File "acl2.ico"           ; copy the icon
  File /R "dist\bin"        ; recursively copy the files in bin
  File /R "dist\etc"        ; recursively copy the files in etc
  File /R "dist\lib"        ; recursively copy the files in lib
  File /R "dist\mingw"      ; recursively copy the files in mingw

  Push "$INSTDIR\bin"       ; add the bin folder to the user's path
  Call AddToPath

  Push "SBCL_HOME"          ; bind SBCL_HOME to instdir\bin
  Push "$INSTDIR\bin"
  Call WriteEnvStr

  Push "$INSTDIR\mingw\bin" ; add the mingw\bin folder to the user's path
  Call AddToPath


  ; Set up the uninstall program and add the program to the add/remove programs
  ; list so that the user can easily choose to uninstall it.

  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ACL2 ${Ver}" \
    "DisplayName" "ACL2 ${VER} (remove only)"

  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\ACL2 ${Ver}" \
    "UninstallString" "$INSTDIR\Uninstall.exe"

  WriteUninstaller "$INSTDIR\Uninstall.exe"


  ; Now we'll set up start menu options.

  CreateShortCut "$DESKTOP\ACL2 ${Ver}.lnk" "$INSTDIR\bin\acl2.exe" \
    "icon.file" "$INSTDIR\acl2.ico"

  CreateDirectory "$SMPROGRAMS\ACL2 ${Ver}"

  CreateShortCut "$SMPROGRAMS\ACL2 ${Ver}\ACL2 ${Ver}.lnk" "$INSTDIR\bin\acl2.exe" \
    "icon.file" "$INSTDIR\acl2.ico"

  !insertmacro CreateInternetShortcut "$SMPROGRAMS\ACL2 ${Ver}\Tour for New Users" \
    "${WebRoot}/A_Flying_Tour_of_ACL2.html" \
    "$INSTDIR\acl2.ico" 0

  !insertmacro CreateInternetShortcut "$SMPROGRAMS\ACL2 ${Ver}\Join the Mailing List" \
    "http://www.cs.utexas.edu/users/moore/acl2/admin/forms/email.html" \
    "$INSTDIR\acl2.ico" 0

  !insertmacro CreateInternetShortcut "$SMPROGRAMS\ACL2 ${Ver}\Order the Textbook" \
    "http://www.cs.utexas.edu/users/moore/publications/acl2-books/car/index.html" \
    "$INSTDIR\acl2.ico" 0

  CreateShortCut "$SMPROGRAMS\ACL2 ${Ver}\Uninstall.lnk" "$INSTDIR\Uninstall.exe" \
     "icon.file" "$INSTDIR\Uninstall.exe" 0

  CreateDirectory "$SMPROGRAMS\ACL2 ${Ver}\Web Resources"

  !insertmacro CreateInternetShortcut "$SMPROGRAMS\ACL2 ${Ver}\Web Resources\ACL2 Homepage" \
    "http://www.cs.utexas.edu/users/moore/acl2/" \
    "$INSTDIR\acl2.ico" 0

  !insertmacro CreateInternetShortcut "$SMPROGRAMS\ACL2 ${Ver}\Web Resources\Documentation (Reference Manual)" \
    "${WebRoot}/acl2-doc-index.html" \
    "$INSTDIR\acl2.ico" 0

  !insertmacro CreateInternetShortcut "$SMPROGRAMS\ACL2 ${Ver}\Web Resources\Papers about ACL2" \
    "http://www.cs.utexas.edu/users/moore/publications/acl2-papers.html" \
    "$INSTDIR\acl2.ico" 0

  !insertmacro CreateInternetShortcut "$SMPROGRAMS\ACL2 ${Ver}\Web Resources\Workshops (More Papers about ACL2)" \
    "http://www.cs.utexas.edu/users/moore/acl2/workshops.html" \
    "$INSTDIR\acl2.ico" 0

SectionEnd



Section "!ACL2 Sources / Documentation" SecSources

  SetOutPath "$INSTDIR"
  File /R "dist\acl2-sources"

  CreateShortCut "$SMPROGRAMS\ACL2 ${Ver}\ACL2 Reference Documentation (Local).lnk" \
    "$INSTDIR\acl2-sources\doc\HTML\acl2-doc-index.html"

SectionEnd



Section "!Standard Books" SecBooks

  SetOutPath "$INSTDIR\acl2-sources"
  File /R "dist\books"

SectionEnd


Section /o "Workshop Books" SecWorkshop

  SetOutPath "$INSTDIR\acl2-sources\books"
  File /R "dist\workshops"

SectionEnd


Section /o "Emacs 21.3 for Windows" SecEmacs

  SetOutPath "$INSTDIR"
  File /R "dist\emacs-21.3"

  CreateShortcut "$SMPROGRAMS\ACL2 ${Ver}\Emacs 21.3.lnk" \ 
    "$INSTDIR\emacs-21.3\bin\runemacs.exe"

SectionEnd




Section "- Post Install Cleanup" SecClean

  SetOutPath "$INSTDIR"
  File post-install.sh
  File post-install.bat
  nsExec::Exec "$INSTDIR\post-install.bat"
  Delete "$INSTDIR\post-install.sh"	
  Delete "$INSTDIR\post-install.bat"

SectionEnd




LangString DESC_SecSources ${LANG_ENGLISH} "Recommended.  Includes all of the ACL2 source code, Emacs extensions, and a local copy of the reference documentation in HTML format and for Emacs."

LangString DESC_SecBooks ${LANG_ENGLISH} "Recommended.  Includes all of the standard ACL2 books on arithmetic, bdds, data structures, set theory, ordinals, powerlists, rtl, etc."

LangString DESC_SecWorkshop ${LANG_ENGLISH} "Optional.  Includes all of the books from the ACL2 workshops from 1999 to 2006."

LangString DESC_SecEmacs ${LANG_ENGLISH} "Optional.  Includes the Emacs text editor, version 21.3, which many ACL2 users prefer for its excellent Lisp support."

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SecSources} $(DESC_SecSources)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecBooks} $(DESC_SecBooks)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecWorkshop} $(DESC_SecWorkshop)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecEmacs} $(DESC_SecEmacs)
!insertmacro MUI_FUNCTION_DESCRIPTION_END


Section "Uninstall"

	  Push "$INSTDIR\bin"
	  Call un.RemoveFromPath

	  Push "$INSTDIR\mingw\bin"
	  Call un.RemoveFromPath

	  RMDir /r "$INSTDIR"    
	  RMDir "$INSTDIR"
	  RMDir /r "$SMPROGRAMS\ACL2 ${Ver}"
	  RMDir "$SMPrograms\ACL2 ${Ver}"

	  Delete "$DESKTOP\ACL2 ${Ver}.lnk"
	  DeleteRegKey HKEY_LOCAL_MACHINE "SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\ACL2 ${Ver}"

SectionEnd